{ Demo program for ChroMathsome - Linear Algebra.
  Below, you find example implementations of using functions available in
  ChroMathsome - Linear Algebra.

  This code was written during the development stages of ChroMathsome.
  There's no pre-written script followed so you may find (some of) this code
  (the order of implemented functions) and some of the variables a little strange.
  
  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Since ChroMathsome - Linear Algebra automatically updates the dimensions of
  matrices and vectors (if needed), you can give most of the matrices/vectors in the
  initialization part (InitRealMatrix, ...) a dummy value.
  You are, however, encouraged to enter the correct values (if you know them at forehand)
  since this will make your code much more readable!

  When studying the code below, notice how nicely ChroMathsome - Linear Algebra
  keeps the user away from the implementation of matrices/vectors (information hiding principle).
  Matrices/Vectors can be used as single datatypes.

  Note that if you use matrices with large dimensions ( > 20*20), the memo (visible interface)
  will become full very fast and the program will stop giving results.
  This is a shortcoming of the memo component (Delphi 1), not of ChroMathsome - Linear Algebra.
  Eventually delete the contents of the memo when it becomes full (or write results to a file).

  Read the help file for information about the implemented functions
  and their limitations/possibilities!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: April 16 1997
  Reader may do with this code whatever he/she wants.

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "A".              }


unit Matrices;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Linalg, Menus, StdCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    Exit1: TMenuItem;
    Clear1: TMenuItem;
    RealMatrix1: TMenuItem;
    ComplexMatrix1: TMenuItem;
    Random1: TMenuItem;
    Special1: TMenuItem;
    Arithmetic1: TMenuItem;
    Build1: TMenuItem;
    Random2: TMenuItem;
    Special2: TMenuItem;
    Arithmetic2: TMenuItem;
    Build2: TMenuItem;
    Identity1: TMenuItem;
    Hilbert1: TMenuItem;
    Identity2: TMenuItem;
    Hilbert2: TMenuItem;
    Add1: TMenuItem;
    Subtract1: TMenuItem;
    Multiply1: TMenuItem;
    ScalarMultiplication1: TMenuItem;
    Add2: TMenuItem;
    Subtract2: TMenuItem;
    Multiply2: TMenuItem;
    ScalarMultiplication2: TMenuItem;
    SwapRows1: TMenuItem;
    SwapColumns1: TMenuItem;
    ReplaceElement1: TMenuItem;
    AddRow1: TMenuItem;
    AddColumn1: TMenuItem;
    DeleteRow1: TMenuItem;
    DeleteColumn1: TMenuItem;
    InsertRow1: TMenuItem;
    InsertColumn1: TMenuItem;
    ReplaceRow1: TMenuItem;
    ReplaceColumn1: TMenuItem;
    SubMatrix1: TMenuItem;
    SeeRealMatrixBuild1: TMenuItem;
    RealVector1: TMenuItem;
    ComplexVector1: TMenuItem;
    Random3: TMenuItem;
    Arithmetic3: TMenuItem;
    Build3: TMenuItem;
    Random4: TMenuItem;
    Arithmetic4: TMenuItem;
    Build4: TMenuItem;
    AddElement1: TMenuItem;
    SwapElements1: TMenuItem;
    ReplaceElements1: TMenuItem;
    InsertElement1: TMenuItem;
    DeleteElement1: TMenuItem;
    SubVector1: TMenuItem;
    SeeComplexVectorBuild1: TMenuItem;
    Vandermonde1: TMenuItem;
    Vandermonde2: TMenuItem;
    Diagonal1: TMenuItem;
    Diagonal2: TMenuItem;
    RowsasVectors1: TMenuItem;
    Dot1: TMenuItem;
    Cross1: TMenuItem;
    Angle1: TMenuItem;
    SeeRealVectorArithmetic1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Random1Click(Sender: TObject);
    procedure SpecialMatrix(Sender: TObject);
    procedure RealMatrixArithmetic(Sender: TObject);
    procedure ComplexMatrixArithmetic(Sender: TObject);
    procedure RealMatrixBuild(Sender: TObject);
    procedure SeeRealMatrixBuild1Click(Sender: TObject);
    procedure RandomVector(Sender: TObject);
    procedure VectorBuild(Sender: TObject);
    procedure SeeComplexVectorBuild1Click(Sender: TObject);
    procedure RowsasVectors1Click(Sender: TObject);
    procedure VectorArithmetic(Sender: TObject);
    procedure SeeRealVectorArithmetic1Click(Sender: TObject);

  private
    { Private declarations }
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);
    procedure Write_RealVector(V: RealVector);
    procedure Write_ComplexVector(V: ComplexVector);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Exit1Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.Write_Message(s: string);
begin
     Memo1.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look.
  Note that to reach a specified element in a matrix/vector, you must use carets (^).}
procedure TForm1.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin  { below, you see the carets }
    if abs(M.Matrix^[a]^[b]) < 1E-10 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,10,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

{ Complex counterpart of above. }
procedure TForm1.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-12 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-12 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    Memo1.Lines.add(mats);
    end;
end;

{ Write a real vector to the screen, creating a nice readable look. }
procedure TForm1.Write_RealVector(V: RealVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    if abs(V.Vector^[a]) < 1E-10 then
    begin
    V.Vector^[a]:= 0;
    matt:= IntToStr(round(V.Vector^[a]));
    end
    else
    matt:= FloatToStrF(V.Vector^[a], ffexponent,5,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;

{ Complex counterpart of above. }
procedure TForm1.Write_ComplexVector(V: ComplexVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    matt:= '(' + FloatToStr(V.Vector^[a].re) + ';' +  FloatToStr(V.Vector^[a].im) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;

procedure TForm1.Clear1Click(Sender: TObject);
begin
     if MessageDlg('Delete the contents of this memo?',
     mtInformation, [mbYes, mbNo], 0) = mrYes
     then
     Memo1.Lines.Clear;
end;

procedure TForm1.Random1Click(Sender: TObject);
var M: RealMatrix;
    CM: ComplexMatrix;
begin
     InitRealMatrix(M, 5, 5);
     InitComplexMatrix(CM, 10, 10);
     try
     if Sender = Random1 then
     begin
     Random_RealMatrix(M, 9);
     Write_Message('A random real matrix:');
     Write_RealMatrix(M);
     end
     else { Complex matrix }
     begin
     Random_ComplexMatrix(CM, 999);
     Write_Message('A random complex matrix:');
     Write_ComplexMatrix(CM);
     end;
     finally
     DestroyRealMatrix(M);
     DestroyComplexMatrix(CM);
     end;
end;

procedure TForm1.SpecialMatrix(Sender: TObject);
var M: RealMatrix;
    CM: ComplexMatrix;
    V: RealVector;
    CV: ComplexVector;
    IsComplex: boolean;
begin
     IsComplex:= false;
     InitRealMatrix(M, 5, 5);
     InitComplexMatrix(CM, 5, 5);
     InitRealVector(V, 5);
     InitComplexVector(CV, 5);
     try
     if Sender = Identity1 then RealIdentityMatrix(M)
     else if Sender = Hilbert1 then RealHilbertMatrix(M)
     else if Sender = Vandermonde1 then
     begin
     Random_RealVector(V, 99);
     Write_RealVector(V);
     Write_Message('Vandermonde matrix: ');
     RealVandermondeMatrix(M, V)
     end
     else if Sender = Diagonal2 then
     begin
     Random_RealVector(V, 99);
     Write_RealVector(V);
     Write_Message('Diagonal matrix: ');
     RealDiagonalMatrix(M, V)
     end
     else
     begin
     IsComplex:= true;
     if Sender = Identity2 then ComplexIdentityMatrix(CM)
     else if Sender = Hilbert2 then ComplexHilbertMatrix(CM)
     else if Sender = Vandermonde2 then
     begin
     Random_ComplexVector(CV, 99);
     Write_ComplexVector(CV);
     Write_Message('Vandermonde matrix: ');
     ComplexVandermondeMatrix(CM, CV)
     end
     else if Sender = Diagonal1 then
     begin
     Random_ComplexVector(CV, 99);
     Write_ComplexVector(CV);
     Write_Message('Diagonal matrix: ');
     ComplexDiagonalMatrix(CM, CV)
     end;
     end;

     Write_Message('Result matrix:');
     if IsComplex = false then Write_RealMatrix(M)
     else Write_ComplexMatrix(CM);
     finally
     DestroyRealMatrix(M);
     DestroyComplexMatrix(CM);
     DestroyRealVector(V);
     DestroyComplexVector(CV);
     end;
end;

procedure TForm1.RealMatrixArithmetic(Sender: TObject);
var M, M1, M2: RealMatrix;
begin
     InitRealMatrix(M, 3, 3);
     InitRealMatrix(M1, 3, 3);
     InitRealMatrix(M2, 3, 3);
     try
     Random_RealMatrix(M1, 10);
     Write_Message('A random real matrix:');
     Write_RealMatrix(M1);
     Random_RealMatrix(M2, 10);
     Write_Message('A random real matrix:');
     Write_RealMatrix(M2);

     if Sender = Add2 then RealMatrix_Addition(M, M1, M2)
     else if Sender = Subtract2 then RealMatrix_Subtraction(M, M1, M2)
     else if Sender = Multiply2 then RealMatrix_Multiplication(M, M1, M2)
     else
     begin
     RealMatrix_RealScalarMultiplication(M1, 666);
     Copy_RealMatrix(M, M1);
     end;

     Write_Message('Result matrix:');
     Write_RealMatrix(M);

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(M1);
     DestroyRealMatrix(M2);
     end;
end;

procedure TForm1.ComplexMatrixArithmetic(Sender: TObject);
var M, M1, M2: ComplexMatrix;
    c: complex;
begin
     InitComplexMatrix(M, 3, 3);
     InitComplexMatrix(M1, 3, 3);
     InitComplexMatrix(M2, 3, 3);
     try
     Random_ComplexMatrix(M1, 10);
     Write_Message('A random complex matrix:');
     Write_ComplexMatrix(M1);
     Random_ComplexMatrix(M2, 10);
     Write_Message('A random complex matrix:');
     Write_ComplexMatrix(M2);

     if Sender = Add1 then ComplexMatrix_Addition(M, M1, M2)
     else if Sender = Subtract1 then ComplexMatrix_Subtraction(M, M1, M2)
     else if Sender = Multiply1 then ComplexMatrix_Multiplication(M, M1, M2)
     else
     begin
     c.re:= 333;
     c.im:= 666;
     ComplexMatrix_ComplexScalarMultiplication(M1, c);
     Copy_ComplexMatrix(M, M1);
     end;

     Write_Message('Result matrix:');
     Write_ComplexMatrix(M);

     finally
     DestroyComplexMatrix(M);
     DestroyComplexMatrix(M1);
     DestroyComplexMatrix(M2);
     end;
end;

{ Functions to build matrices.
  Note that for all these functions ChroMathsome - Linear Algebra automatically updates
  the dimensions of the matrix (if needed) -> elegant, readable and understandable code ;-) }
procedure TForm1.RealMatrixBuild(Sender: TObject);
var M: RealMatrix;
    V: RealVector;
    r, c: integer;
begin
     InitRealMatrix(M, 5, 5);
     InitRealVector(V, 5);
     try
     Random_RealMatrix(M, 10);
     Write_Message('A random real matrix:');
     Write_RealMatrix(M);
     Random_RealVector(V, 10);
     Write_Message('A random real vector:');
     Write_RealVector(V);
     r:= random(M.Row) + 1; { May not be zero! }
     c:= random(M.Col) + 1; { May not be zero! }

     if Sender = SwapRows1 then RealMatrix_SwapRows(M, r, c)
     else if Sender = SwapColumns1 then RealMatrix_SwapColumns(M, r, c)
     else if Sender = ReplaceElement1 then RealMatrix_ReplaceElement(M, r, c, 999)
     else if Sender = AddRow1 then RealMatrix_AddRow(M, V)
     else if Sender = AddColumn1 then RealMatrix_AddColumn(M, V)
     else if Sender = DeleteRow1 then RealMatrix_DeleteRow(M, r)
     else if Sender = DeleteColumn1 then RealMatrix_DeleteColumn(M, c)
     else if Sender = InsertRow1 then RealMatrix_InsertRow(M, V, r)
     else if Sender = InsertColumn1 then RealMatrix_InsertColumn(M, V, c)
     else if Sender = ReplaceRow1 then RealMatrix_ReplaceRow(M, V, r)
     else if Sender = ReplaceColumn1 then RealMatrix_ReplaceColumn(M, V, c);
    { else if Sender = SubMatrix1 then RealSubMatrix(M, M, r, c);}

     Write_Message('Row: ' + IntToStr(r) + ' Column: ' + IntToStr(c));
     Write_Message('Result matrix:');
     Write_RealMatrix(M);

     finally
     DestroyRealMatrix(M);
     DestroyRealVector(V);
     end;
end;

procedure TForm1.SeeRealMatrixBuild1Click(Sender: TObject);
begin
     ShowMessage('Look at RealMatrix/Build.');
end;

procedure TForm1.RandomVector(Sender: TObject);
var V: RealVector;
    CV: ComplexVector;
begin
     InitRealVector(V, 5);
     InitComplexVector(CV, 5);
     try
     if Sender = Random3 then
     begin
     Random_RealVector(V, 999);
     Write_Message('A random real vector:');
     Write_RealVector(V);
     end
     else { Complex Vector }
     begin
     Random_ComplexVector(CV, 999);
     Write_Message('A random complex vector:');
     Write_ComplexVector(CV);
     end;
     finally
     DestroyRealVector(V);
     DestroyComplexVector(CV);
     end;
end;

{ See remark RealMatrixBuild. }
procedure TForm1.VectorBuild(Sender: TObject);
var c: complex;
    index1, index2: integer;
    V: ComplexVector;
begin
     InitComplexVector(V, 5);
     try
     Random_ComplexVector(V, 999);
     Write_Message('A random complex vector:');
     Write_ComplexVector(V);

     index1:= random(V.Dimension) + 1; { May not be zero! }
     index2:= random(V.Dimension) + 1; { May not be zero! }
     Write_Message('Index1: ' + IntToStr(index1) + ' Index2: ' + IntToStr(index2));
     c.re:= 999;
     c.im:= 111;

     if Sender = AddElement1 then ComplexVector_AddElement(V, c)
     else if Sender = SwapElements1 then ComplexVector_SwapElements(V, index1, index2)
     else if Sender = ReplaceElements1 then ComplexVector_ReplaceElement(V, index1, c)
     else if Sender = InsertElement1 then ComplexVector_InsertElement(V, index1, c)
     else if Sender = DeleteElement1 then ComplexVector_DeleteElement(V, index1)
     else if Sender = SubVector1 then ComplexSubVector(V, V, index1, index2);

     Write_Message('Result vector:');
     Write_ComplexVector(V);
     finally
     DestroyComplexVector(V);
     end;
end;

procedure TForm1.SeeComplexVectorBuild1Click(Sender: TObject);
begin
     ShowMessage('Look at ComplexVector/Build.');
end;

procedure TForm1.RowsasVectors1Click(Sender: TObject);
var M: RealMatrix;
    V: RealVector;
    i: integer;
begin
     InitRealMatrix(M, 10, 10);
     InitRealVector(V, 10);
     try
     Random_RealMatrix(M, 99);
     Write_Message('Original matrix: ');
     Write_RealMatrix(M);
     for i:= 1 to M.Row do
     begin
     Write_Message('Row ' + IntToStr(i) + ' as vector: ');
     RealMatrixRowToRealVector(V, M, i);
     Write_RealVector(V);
     end;

     finally
     DestroyRealMatrix(M);
     DestroyRealVector(V);
     end;
end;

procedure TForm1.VectorArithmetic(Sender: TObject);
var V, V1, V2: RealVector;
    x: extended;
begin
     InitRealVector(V, 3);
     InitRealVector(V1, 3);
     InitRealVector(V2, 3);
     try
     Random_RealVector(V1, 100);
     Random_RealVector(V2, 10);
     Write_Message('A random vector: ');
     Write_RealVector(V1);
     Write_Message('Another random vector: ');
     Write_RealVector(V2);

     if ((Sender = Dot1) or (Sender = Angle1)) then
     begin
     if Sender = Dot1 then x:= RealVector_DotProduct(V1, V2)
     else x:= RealVector_Angle(V1, V2); { Angle in radians! }
     Write_Message('Result: ' + FloatToStr(x));
     end
     else { Cross product of 2 3-dimensional vectors. }
     begin
     RealVector_CrossProduct(V, V1, V2);
     Write_Message('Result: ');
     Write_RealVector(V);
     end;

     finally
     DestroyRealVector(V);
     DestroyRealVector(V1);
     DestroyRealVector(V2);
     end;
end;

procedure TForm1.SeeRealVectorArithmetic1Click(Sender: TObject);
begin
     ShowMessage('Look at RealVector/Arithmetic.');
end;

end.
